package com.gree.mall.manager.logic.policy;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.policy.*;
import com.gree.mall.manager.commonmapper.PolicyCMapper;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class AgreementLogic {

    @Autowired
    PolicyCMapper policyCMapper;


    @Autowired
    CommonLogic commonLogic;

    @Autowired
    AgreementService agreementService;

    @Autowired
    AgreementPolicyService agreementPolicyService;

    @Autowired
    PolicyService policyService;

    @Autowired
    MailboxService mailboxService;

    @Autowired
    MailboxSendRecordService mailboxSendRecordService;



    public IPage<AgreementVo> list(Page page, ZfireParamBean zfireParam) {

        //获取当前登录企业id
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<String> adminCompanyIds = adminUser.getCompanyWechatIds();
        //1.组装查询条件
        zfireParam = FieldUtils.supplyParam(zfireParam, PolicyVo.class);

        IPage<AgreementVo> agreementVoIPage = policyCMapper.listAgreement(page, zfireParam,adminCompanyIds);
        return agreementVoIPage;
    }

    public AgreementDetail detail(String id) {

        Agreement agreement = agreementService.getById(id);

        AgreementDetail agreementDetail = BeanUtil.toBean(agreement, AgreementDetail.class);

        List<AgreementPolicy> agreementPolicies = agreementPolicyService.lambdaQuery().eq(AgreementPolicy::getAgreementId, agreement.getId()).list();


        List<String> aggre = agreementPolicies.stream().map(AgreementPolicy::getAgreementId).collect(Collectors.toList());


        List<Policy> policyList = policyService.lambdaQuery().in(Policy::getId, aggre).list();

        agreementDetail.setAgreementPolicies(agreementPolicies);
        agreementDetail.setPolicy(policyList);

        return agreementDetail;

    }

    @Transactional(rollbackFor = Exception.class)
    public void add(AgreementAddUp agreementAddUp) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        agreementAddUp.setCompanyWechatId(adminUser.getCompanyWechatId());
        agreementAddUp.setCompanyWechatName(adminUser.getCompanyName());
        agreementAddUp.insert();

        for (AgreementPolicy agreementPolicy : agreementAddUp.getAgreementPolicies()) {
            agreementPolicy.setAgreementId(agreementAddUp.getId());
        }
        agreementPolicyService.saveBatch(agreementAddUp.getAgreementPolicies());
    }


    @Transactional(rollbackFor = Exception.class)
    public void update(AgreementAddUp agreementAddUp) {


        agreementAddUp.updateById();

        agreementPolicyService.lambdaUpdate().eq(AgreementPolicy::getAgreementId,agreementAddUp.getId()).remove();

        for (AgreementPolicy agreementPolicy : agreementAddUp.getAgreementPolicies()) {
            agreementPolicy.setAgreementId(agreementAddUp.getId());
        }
        agreementPolicyService.saveBatch(agreementAddUp.getAgreementPolicies());
    }

    public Mailbox detailMailbox() {

        if (mailboxService.lambdaQuery().eq(Mailbox::getId,"1").count() < 1)
            return new Mailbox();


        return mailboxService.getById("1");

    }

    public void addDetail(Mailbox mailbox) {
        if (mailboxService.lambdaQuery().eq(Mailbox::getId,"1").count() < 1){
            mailbox.setId("1");
            mailbox.insert();
            return;
        }
        mailbox.setId("1");
        mailbox.updateById();

    }

    public IPage<MailBoxSendRecordVo> listMailBoxRecord(Page page, ZfireParamBean zfireParam) {
        //获取当前登录企业id
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<String> adminCompanyIds = adminUser.getCompanyWechatIds();
        //1.组装查询条件
        zfireParam = FieldUtils.supplyParam(zfireParam, MailBoxSendRecordVo.class);

        IPage<MailBoxSendRecordVo> policyVoIPage = policyCMapper.listMailBoxRecord(page, zfireParam,adminCompanyIds);
        return policyVoIPage;
    }

    public MailboxSendRecord detailRecord(String id) {
        MailboxSendRecord mailboxSendRecord = mailboxSendRecordService.getById(id);
        return mailboxSendRecord;
    }

    public IPage<MailboxSendRecordPolicyVo> listPolicy(Page page, ZfireParamBean zfireParam) {
        //获取当前登录企业id
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<String> adminCompanyIds = adminUser.getCompanyWechatIds();
        //1.组装查询条件
        zfireParam = FieldUtils.supplyParam(zfireParam, MailboxSendRecordPolicyVo.class);

        IPage<MailboxSendRecordPolicyVo> policyVoIPage = policyCMapper.listPolicy(page, zfireParam,adminCompanyIds);
        return policyVoIPage;
    }
}
